<?php

namespace Controller;

use Service\Traits\TResponse;
use Service\DeleteCascadeService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Util\Utilities;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;
use Zeedhi\Framework\DataSource\FilterCriteria;

class DefaultColumns extends Crud{
    
    use TResponse;

    protected $dataSourceName = "defaultColumns";

    protected $defaultColumnsService;

    public function __construct(Manager $mongoManager, DeleteCascadeService $deleteCascadeService, \Service\DefaultColumns $defaultColumnsService)
    {
        parent::__construct($mongoManager, $deleteCascadeService);
        $this->defaultColumnsService = $defaultColumnsService;
        $this->mongoManager->createDatabase(\Util\Utilities::getOrganizationIdForMongo());
    }

    public function findNext(DTO\Request $request, DTO\Response $response)
    {
        try {
            $params = $request->getParameters();
            $filter = $this->defaultColumnsService->validateFilter($params);
            $defaultColumns = Utilities::parseDataSourceZeedhiNext($this->defaultColumnsService->getDefaultColumns($filter));
            $defaultColumns['data'] = $this->fillEmptyAttributes($defaultColumns['data']);
            $defaultColumns['pagination']['total'] = $this->defaultColumnsService->count($filter);
            $defaultColumns['pagination']['limit'] = $params['limit'];
            $defaultColumns['pagination']['page'] = $params['page'];
            $this->sendResponse($defaultColumns, 200);
            //$response->addDataSet(new DataSet('defaultColumns', $defaultColumns));
        } catch (Exception $e) {
        }
    }

    public function updateOrInsertDefault(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            $persistedRows = $this->defaultColumnsService->updateOrInsertDefaultColumn($row);
            $nrPersistedRows = count($persistedRows);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
            $response->addNotification(new DTO\Response\Notification("{$nrPersistedRows} columns subscribed with success"));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    private function fillEmptyAttributes($defaultColumns)
    {
        foreach ($defaultColumns as $key => $defaultColumn) {
            $defaultColumns[$key]['type'] = array_key_exists('type', $defaultColumn) ? $defaultColumn['type'] : '';
        }

        return $defaultColumns;
    }

    private function formatServerQuery($serverQuery)
    {
        $paramsString = explode('&', $serverQuery);
        $params = array();
        if ($paramsString && $paramsString[0]) {
            foreach ($paramsString as $key => $value) {
                $parts = explode('=', $value);
                if (strpos($parts[0], '%5B') !== false) {
                    $newParts = explode('%5B', $parts[0]);
                    $parts[0] = $newParts[0];
                }
                if ($parts[1] == "true") $parts[1] = true;
                else if ($parts[1] == "false") $parts[1] = false;
                if (!isset($params[$parts[0]])) {
                    $params[$parts[0]] = $parts[1];
                } else {
                    if (!is_array($params[$parts[0]])) $params[$parts[0]] = array($params[$parts[0]]);
                    array_push($params[$parts[0]], $parts[1]);
                }
            }
        }

        return $params;
    }

    private function comparatorAsc($object1, $object2)
    {
        return strcmp(strtolower($object1[$this->orderField]), strtolower($object2[$this->orderField]));
    }

    private function comparatorDesc($object1, $object2)
    {
        return strcmp(strtolower($object2[$this->orderField]), strtolower($object1[$this->orderField]));
    }
}
