<?php
namespace Controller;

use Service\Traits\TResponse;
use Service\Manager\Mongo\Manager;
use Util\Utilities;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Service\ImportDefault as ImportDefaultService;

class ImportDefault extends Crud {
    
    use TResponse;
    
    protected $mongoManager;
    protected $importService;

    public function __construct(){

        $mongoDbName = InstanceManager::getInstance()->getParameter('mongoDBNamePrefix');
        $host = InstanceManager::getInstance()->getParameter('mongoHost');
        $port = InstanceManager::getInstance()->getParameter('mongoPort');
        $baseConnections = array(array(
            'host' => $host,
            'port' => $port,
            'username' => 'zmartzeedhi',
            'password' => 'zmartzeedhi',
            'dbName' => $mongoDbName . '5',
            'dbLabel' => 'Teknisa'
        ));
        Utilities::getEnvironment()->setBaseConnection($baseConnections);

        $this->mongoManager = InstanceManager::getInstance()->getService('organizationMongoManager');
        $this->mongoManager->createDatabase(\Util\Utilities::getOrganizationIdForMongo());
        $this->importService = InstanceManager::getInstance()->getService('\Service\ImportDefault');
	}

    public function findImportNext(DTO\Request $request, DTO\Response $response) {
        $filter = Utilities::parseParamNextToZmartFilterCriteria("findImport", $request->getParameters());
        $importData = $this->importService->findImportData($filter->getConditions());
        $this->sendResponse(Utilities::parseDataSourceZeedhiNext($importData), 200);
    }

    public function importDefault(DTO\Request $request, DTO\Response $response){
        $rows = $request->getRow();
        $accessProfiles = !empty($rows['accessProfiles']) ? $rows["accessProfiles"] : null;
        $row = $rows["rows"];
        list($importedItems,$errorMessages) = $this->importService->importDefault($row, $accessProfiles);
        $response->addDataSet(new DataSet('importDefault', $importedItems));
        $this->importService->setLogImport($importedItems['dashboard'],$importedItems['indicator'],$importedItems['dataSource']);
        foreach($importedItems as $itemName => $itemCount){
            if(count($itemCount) > 0){
                $response->addNotification(new DTO\Response\Notification(Exception::buildErrorJSON(
                    "PERSISTED_SUCCESSFULLY",
                    array("count" => count($itemCount), "name" => $itemName)
                )));
            }
        }
        foreach($errorMessages as $error){
            $response->addNotification(new DTO\Response\Notification($error,"error"));
        }
    }
    
    public function findImportIndicatorViewsNext(DTO\Request $request, DTO\Response $response) {
        $filter = Utilities::parseParamNextToZmartFilterCriteria("findImportIndicatorViews", $request->getParameters());
        $conditions = $filter->getConditions();
        //  $conditions = $request->getFilterCriteria()->getConditions();
        //  $indicator  = Utilities::getValueInFilterDataSourceByName($conditions, 'indicator', true);
        $importData = $this->importService->findImportIndicatorViews($conditions);
        $parsedData = Utilities::parseDataSourceZeedhiNext($importData);
        $parsedData = $this->implementPaginationNext($parsedData, $filter);
        $this->sendResponse($parsedData, 200);
    }
    
    public function implementPaginationNext($zeedhiNextData, $filter) {
        if (!empty($filter->getPage()) && !empty($filter->getPageSize()) && $filter->getPageSize() > 0) {
            $data = $zeedhiNextData["data"];
            $page = $filter->getPage();
            $limit = $filter->getPageSize();
            
            if (count($data) > 0) {
                $data = array_chunk($data, $limit);
                if ($page > count($data))
                    $page = count($data);
                if ($page <= 0)
                    $page = 1;
                $zeedhiNextData["data"] = $data[$page - 1];
            }
            
            $zeedhiNextData["pagination"]["page"] = $page;
            $zeedhiNextData["pagination"]["limit"] = $limit;
        }
        return $zeedhiNextData;
    }
}
