<?php
namespace Service\IntegrationOM;

use Util\Environment;
use Util\InstanceProvider;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;

class ApiWebService
{

    public static function getStructureType($allStructureType,$id){
        $result = null;
        foreach ( $allStructureType as $structureType ) {
            if($structureType['_id'] == $id){
                $result = $structureType;
            }
        }
        return $result;
    }

    public function getVariableMeasureQuantity($nrorg,$contractId,$dtinicompet ,$dtfincompet){
       list( $result ) = $this->ExecVariableMeasure($nrorg);
        $count = count($result);
        if(empty($result)){
            $contract = self::findContract($contractId,$dtinicompet ,$dtfincompet);
            if(!empty($contract[0])){
                $count = $contract[0]['QTMEDIDAVENDIDA'];
            }
        }
        $dataset = array('COUNT' => $count);
        echo json_encode($dataset);
    }

    public function getVariableMeasureLog($nrorg,$contractId,$dtinicompet ,$dtfincompet){
        list( $result,$structureACL,$structureUser )  = $this->ExecVariableMeasure($nrorg);

        $dataset = self::createLog($result,$structureACL,$structureUser);
        if(empty($dataset)){
            $contract = self::findContract($contractId,$dtinicompet ,$dtfincompet);
            if(!empty($contract)){
                $row = array();
                $row["Nome"] 			= 'Não foram encontrados dados a serem coletados. Considerando a quantidade mínima do contrato';
                $dataset = array($row);
            }else{
                $row = array();
                $row["Nome"] 			= 'Contrato não encontado';
                $row["NRSEQCONTRATO"]   = $contractId;
                $row["DTINICOMPET"]     = $dtinicompet;
                $row["DTFINCOMPET"]     = $dtfincompet;
                $row["NRPRODUTO_ID"]    = InstanceManager::getInstance()->getParameter('projectId');;
                $dataset = array($row);
            }
        }
        echo json_encode($dataset);
    }
    public static function createLog ($result,$structureACL,$structureUser){
        $dataset = array();
        foreach ($result as &$operador) {
            $strucUser = [];
            if(isset($structureUser[$operador['CDOPERADOR']])){
                $strucUser = $structureUser[$operador['CDOPERADOR']];
            }
            if($operador['NRPERFILACESSO']){
                $operador['NRPERFILACESSO'] = explode(',',$operador['NRPERFILACESSO']);
                foreach($operador['NRPERFILACESSO'] as $nrperfilacesso){
                    if(isset($structureACL[$nrperfilacesso])){
                        if(empty($strucUser)){
                            $strucUser = $structureACL[$nrperfilacesso];
                        }else{
                            $strucUser = array_merge($strucUser,$structureACL[$nrperfilacesso]);
                        }
                    }
                }
            }

            if(!empty($strucUser)) {
                $row = array();
                $row["Código"] 				  = $operador['CDOPERADOR'];
                $row["Nome"] 				  = $operador['NMOPERADOR'];
                $row["Estruturas Gerenciais"] = $strucUser ? implode(' | ', $strucUser) : '';
                $dataset[] = $row;
            }
        }
        return $dataset;
    }

    public static function findContract($contractId,$dtinicompet ,$dtfincompet){
        $productId = InstanceManager::getInstance()->getParameter('projectId');
        $query = "SELECT MEDVAR.IDELEMENTORAIZ, MEDVAR.QTMEDIDAVENDIDA,
                       MED.NRSEQCONTRATO, TO_CHAR(MED.DTINICOMPET, 'DD/MM/YYYY') AS DTINICOMPET, TO_CHAR(MED.DTFINCOMPET, 'DD/MM/YYYY') AS DTFINCOMPET
                  FROM FCT_MEDICAOFATURA MED
                  JOIN FCT_MEDVARMEDICAOFATURA MEDVAR
                    ON MED.NRMEDICAOFATURA = MEDVAR.NRMEDICAOFATURA
                 WHERE MED.NRSEQCONTRATO = :NRSEQCONTRATO
                   AND TO_CHAR(MED.DTINICOMPET, 'DD/MM/YYYY') = :DTINICOMPET
                   AND TO_CHAR(MED.DTFINCOMPET, 'DD/MM/YYYY') = :DTFINCOMPET
                   AND MEDVAR.IDELEMENTORAIZ = (SELECT 'P_' || ID as ID
                                                  FROM OM_PRODUTO
                                                 WHERE NREXTERNAL_PRODUCT_ID = :NRPRODUTO_ID)";
        $param = array(
            'NRSEQCONTRATO' => $contractId,
            'DTINICOMPET'=>$dtinicompet,
            'DTFINCOMPET'=>$dtfincompet,
            'NRPRODUTO_ID'=>$productId,
        );
        return InstanceProvider::getConnectionCustom()->updateVpdConnection(20, true)->getConnection()->fetchAll($query,$param);
    }

    public function ExecVariableMeasure($nrorg)
    {
        $organizationMongoManager = $this->getOrganizationMongoManager($nrorg);

        $found = $this->findMongo($organizationMongoManager,'structure');
        $structureType = $this->findMongo($organizationMongoManager,'structureType');

        list($accessProfiles, $usersResponsible, $structureUser, $structureACL) = $this->getVariableMeasureMongo($found,
            $structureType);

        list($accessProfilesString, $usersResponsibleString) = $this->getStringFilterOracle($accessProfiles, $usersResponsible);

        $result = $this->getVariableMeasureOracle($nrorg, $usersResponsibleString, $accessProfilesString);
        return array($result,$structureACL,$structureUser);
    }

    /**
     * @param $nrorg
     * @return \Service\Manager\Mongo\Manager
     * @throws \Exception
     */
    public function getOrganizationMongoManager($nrorg): \Service\Manager\Mongo\Manager
    {
        $session = \Teknisa\Libs\Util\InstanceProvider::getSession();
        $session->set('NRORG', $nrorg);
        $session->set('organizationId', $nrorg);
        /** @var Environment $envoronment */
        $envoronment = InstanceManager::getInstance()->getService('environment');
        $envoronment->setOrganizationId($nrorg);
        /** @var \Service\Manager\Mongo\Manager $organizationMongoManager */
        $organizationMongoManager = InstanceManager::getInstance()->getService('organizationMongoManager');
        return $organizationMongoManager;
    }


    public function findMongo(\Service\Manager\Mongo\Manager $organizationMongoManager,$collectionName): array
    {
        $filterCriteria = new ZmartFilterCriteria($collectionName, array());
        $filterCriteria->setPageSize(100000);
        $filterCriteria->setPage(1);
        return $organizationMongoManager->simpleFindPagination($filterCriteria);
    }


    public function getVariableMeasureMongo(array $found, array $structureType, $accessProfiles = array(), array $structureACL= array(),
                                            array $usersResponsible= array(), array $structureUser= array(), array $result= array()): array
    {

        foreach ($found as $doc) {
            if (isset($doc['accessProfiles']) && is_array($doc['accessProfiles'])) {
                $accessProfiles = array_merge($accessProfiles, $doc['accessProfiles']);
                if (isset($doc['type'])) {
                    $structure = self::getStructureType($structureType, $doc['type']);
                    if (isset($structure['name'])) {
                        $structureName = $structure['name'] . ' => ' . $doc['name'];
                    }
                }
                if (isset($structureName)) {
                    foreach ($doc['accessProfiles'] as $accessProfile) {
                        if (isset($structureACL[$accessProfile])) {
                            if (!in_array($structureName, $structureACL[$accessProfile])) {
                                $structureACL[$accessProfile][] = $structureName;
                            }
                        } else {
                            $structureACL[$accessProfile] = [$structureName];
                        }

                    }
                }
            }
            if (isset($doc['usersResponsible']) && is_array($doc['usersResponsible'])) {
                $usersResponsible = array_merge($usersResponsible, $doc['usersResponsible']);
                if (isset($doc['type'])) {
                    $structure = self::getStructureType($structureType, $doc['type']);
                    if (isset($structure['name'])) {
                        $structureName = $structure['name'] . ' => ' . $doc['name'];
                    }
                }
                if ($structureName) {
                    foreach ($doc['usersResponsible'] as $user) {
                        if (isset($structureUser[$user])) {
                            if (!in_array($structureName, $structureUser[$user])) {
                                $structureUser[$user][] = $structureName;
                            }
                        } else {
                            $structureUser[$user] = [$structureName];
                        }
                    }
                }
            }
            $result[] = $doc;
        }
        return array($accessProfiles, $usersResponsible, $structureUser, $structureACL);
    }

    /**
     * @param $accessProfiles
     * @param $usersResponsible
     * @return array
     */
    public function getStringFilterOracle($accessProfiles, $usersResponsible): array
    {
        $accessProfilesString = '0';
        $usersResponsibleString = "'0'";
        if (!empty($accessProfiles)) {
            $accessProfilesString = implode(',', $accessProfiles);
        }
        if (!empty($usersResponsible)) {
            $usersResponsibleString = implode("','", $usersResponsible);
            $usersResponsibleString = "'" . $usersResponsibleString . "'";
        }
        return array($accessProfilesString, $usersResponsibleString);
    }


    public function getVariableMeasureOracle($nrorg, $usersResponsibleString, $accessProfilesString)
    {
        $query = "SELECT CDOPERADOR, NMOPERADOR,
				 rtrim(xmlagg(xmlelement(e,NRPERFILACESSO,',').extract('//text()') ORDER BY NRPERFILACESSO).GetClobVal(),',') AS NRPERFILACESSO 
			FROM (SELECT OPE.CDOPERADOR, OPE.NMOPERADOR,PER.NRPERFILACESSO
                    FROM OPERADOR OPE
                    JOIN PERFACESSPARC PER ON (PER.NRPARCNEGOCIO = OPE.NRPARCNEGOCIO)
                   WHERE OPE.NRORG = $nrorg
                     AND OPE.IDOPERATIVO = 'A'
                     AND (OPE.CDOPERADOR IN ($usersResponsibleString)
                      OR PER.NRPERFILACESSO IN ($accessProfilesString))
                GROUP BY OPE.CDOPERADOR, OPE.NMOPERADOR,PER.NRPERFILACESSO)
        GROUP BY CDOPERADOR, NMOPERADOR";

        $result = InstanceProvider::getConnectionCustom()->updateVpdConnection($nrorg, true)->getConnection()->fetchAll($query);
        return $result;
    }

}